<?php

namespace Drupal\if_then_else\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\if_then_else\Event\IfthenelseNodesGenerate;

/**
 * Subscribes to Symfony events and maps them to Rules events.
 */
class FormAlterIfthenelseNodesSubscriber implements EventSubscriberInterface {

  /**
   * Constructor.
   */
  public function __construct() {

  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      // Static class constant => method on this class.
      IfthenelseNodesGenerate::EVENT_NAME => 'onProcessIfthenelseNodes',
    ];
  }

  /**
   * Subscribe to ifthenelseNodes generation.
   *
   * @param \Drupal\ifthenelse\Event\IfthenelseNodesGenerate $event
   *   Event dispatcher event.
   */
  public function onProcessIfthenelseNodes(IfthenelseNodesGenerate $event) {
    $class = get_class();
    $event->ifthenelseNodes[$class] = [
    // Event name of node should be function name except hook_.
      'event' => 'form_alter',
      'library' => 'if_then_else/form_alter',
    ];

    // Calling custom service for ifthenelse utilities.
    $ifthenelseUtilities = \Drupal::service('ifthenelse.utilities');
    $content_entity_types = $ifthenelseUtilities->getContentEntitiesAndBundles();

    // Sending content Entity and bundle list as javascript variable.
    $form_entity_info = json_encode($content_entity_types);

    // This code will execute on update of rule. Will check for
    // presence of form_alter event and presenece of data.
    $form_fields = FALSE;
    $listFields = $ifthenelseUtilities->getFieldsByEntityBundleId($content_entity_types);

    // Sending list of fields as javascript variable.
    if (isset($listFields)) {
      $form_fields = json_encode($listFields);
    }

    // Adding classes and names for nodes.
    $nodes['formAlterEvent'] = [
      'class' => 'Drupal\\if_then_else\\core\\Events\\HookFormAlterEvent\\HookFormAlterEvent',
      'type' => 'event',
      'name' => 'form_alter',
      'library' => 'if_then_else/HookFormAlterEvent',
    ];

    $nodes['formIdCondition'] = [
      'class' => 'Drupal\\if_then_else\\core\\Conditions\\IfthenelseFormAlterCondition\\IfthenelseFormAlterCondition',
      'type' => 'condition',
      'library' => 'if_then_else/IfthenelseFormAlterCondition',
    ];

    $nodes['setFieldValueAction'] = [
      'class' => 'Drupal\\if_then_else\\core\\Actions\\SetFieldDefaultValue\\SetFieldDefaultValue',
      'type' => 'action',
      'library' => 'if_then_else/SetFieldDefaultValue',
    ];

    $nodes['makeFieldsRequiredAction'] = [
      'class' => 'Drupal\\if_then_else\\core\\Actions\\IfthenelseFieldsRequiredAction\\IfthenelseFieldsRequiredAction',
      'type' => 'action',
      'library' => 'if_then_else/IfthenelseFieldsRequiredAction',
    ];

    $nodes['isFieldValueEmpty'] = [
      'class' => 'Drupal\\if_then_else\\core\\Actions\\IfthenelseIsFieldValueEmptyCondition\\IfthenelseIsFieldValueEmptyCondition',
      'type' => 'action',
      'library' => 'if_then_else/IfthenelseIsFieldValueEmptyCondition',
    ];

    $event->ifthenelseNodes[$class]['drupalSettings'] = [
      'form_fields' => $form_fields,
      'entity_info' => $form_entity_info,
      'nodes' => $nodes,
    ];

    foreach ($nodes as $c => $node) {
      if (isset($node['library'])) {
        $event->ifthenelseNodes[$c]['library'] = $node['library'];
        unset($nodes[$c]['library']);
      }
    }

    return $event;
  }

}
