//Vuejs control for action node of making fields required.
var VueFieldsControl = {
  props: ['emitter', 'ikey', 'getData', 'putData'],
  components: {
    Multiselect: window.VueMultiselect.default
  },
  template: '<div class="fields-container"><div class="form-fields-selection" ><multiselect v-model="value" :show-labels="false" tag-placeholder="Add this as new tag" placeholder="Select Entity" label="name" track-by="code" :options="options" :multiple="true" :taggable="true" @input="updateSelected" @tag="addTag"></multiselect></div></div>',
  data() {
    return {
      type: drupalSettings.ifthenelserule.nodes.makeFieldsRequiredAction.type,
      class: drupalSettings.ifthenelserule.nodes.makeFieldsRequiredAction.class,
      value: [],
      options: [],
      form_fields: [],
    }
  },
  methods: {
    addTag (newTag) {
      //Multiselect tags
      const tag = {
        name: newTag,
        code: newTag.substring(0, 2) + Math.floor((Math.random() * 10000000))
      }
      this.options.push(tag)
      this.value.push(tag)				
    },
    updateSelected(value){
      //Triggered when changing field values
      var selectedOptions = [];
      value.forEach((resource) => {
        selectedOptions.push({name: resource.name, code: resource.code});
      })
      this.putData('form_fields',selectedOptions);
      this.emitter.trigger('process');
    }
  },
  mounted() {
    //initialize variable for data
    this.putData('type',drupalSettings.ifthenelserule.nodes.makeFieldsRequiredAction.type);
    this.putData('class',drupalSettings.ifthenelserule.nodes.makeFieldsRequiredAction.class);
    
    //setting values of selected fields when rule edit page loads.
    var get_form_fields = this.getData('form_fields');
    if(typeof get_form_fields != 'undefined'){
      this.value = this.getData('form_fields');
    }else{
      this.putData('form_fields',[]);
    }
    
  },
  created() {
    if(drupalSettings.ifthenelserule.form_fields){
      //setting list of all fields for a form when rule edit page loads.
      this.options = JSON.parse(drupalSettings.ifthenelserule.form_fields);
    }
  }
}

class FieldsControl extends Rete.Control {
  constructor(emitter, key) {
    super(key);
    this.component = VueFieldsControl;
    this.props = { emitter, ikey: key };
  }

  //setting list value of fields. Used when changing entity or bundle value in condition node.
  setOptions(options) {
    this.vueContext.options = options;
  }
  
  //resetting selected fields value when changing the entity or bundle value in codition node.
  setValue(value){
    this.vueContext.value = value;
  }
}

class FieldsRequiredComponent extends Rete.Component {

  constructor(){
      super("Action: Make Fields Required");
  }

  builder(node) {
      var conditionInput = new Rete.Input('execute', "", conditionSocket);
      var formInput = new Rete.Input('form', "", formObjectSocket);

      node.addControl(new FieldsControl(this.editor, 'makeFieldsRequired'));

      return node.addInput(conditionInput).addInput(formInput);
  }

  worker(node, inputs, outputs) {
    outputs['form_fields'] = node.data.value;
  }
}

var components = [new FieldsRequiredComponent()];	

components.map(c => {
  editor.register(c);
  engine.register(c);
});