<?php

namespace Drupal\if_then_else\core\Actions\IfthenelseFieldsRequiredAction;

use Drupal\if_then_else\core\IfthenelseActionInterface;

/**
 * Class defined to execute make fields required action node.
 */
class IfthenelseFieldsRequiredAction implements IfthenelseActionInterface {

  /**
   * All node Inputs.
   *
   * @var array
   */
  public $inputs;

  /**
   * All node Outputs.
   *
   * @var array
   */
  private $output;

  /**
   * Current node object data.
   *
   * @var array
   */
  private $data;

  /**
   * Ifthenelserule constructor.
   *
   * @param object $data
   *   Node data object.
   */
  public function __construct($data) {
    $this->data = $data;
  }

  /**
   * Function to set inputs.
   *
   * @param array $inputs
   *   Array of all inputs to node.
   */
  public function setInputs(array $inputs) {
    $this->inputs = $inputs;
  }

  /**
   * Get node output.
   *
   * @return output
   *   Returns output of current node set by process function.
   */
  public function getOutput() {
    return $this->output;
  }

  /**
   * Process inputs and set output.
   */
  public function process() {
    $data = $this->data;
    $this->makeSingleFieldsRequired($data->form_fields, $this->inputs['form']);
  }

  /**
   * Make selected fields required.
   *
   * @param array $form_fields
   *   List of selected fields to make required.
   * @param object $form
   *   Form object from hook form alter.
   */
  public function makeSingleFieldsRequired(array $form_fields, &$form) {
    foreach ($form_fields as $field) {
      // Field code is field machine name.
      $field_code = $field->code;
      if (isset($form[$field_code])) {
        if ($form[$field_code]['#type'] == 'container') {
          if (isset($form[$field_code]['widget'])) {
            if (isset($form[$field_code]['widget']['#type'])) {
              if (isset($form[$field_code]['widget']['#type']) == 'select') {
                $form[$field_code]['widget']['#required'] = TRUE;
              }
            }
            else {
              foreach ($form[$field_code]['widget'] as $k => $value) {
                if (strpos($k, '#') !== FALSE) {
                  // Skip all keys which have #.
                  continue;
                }

                $form[$field_code]['widget'][$k]['#required'] = TRUE;
              }
            }
          }
        }
        else {
          $form[$field_code]['#required'] = TRUE;
        }
      }
    }
  }

}
