// Text input field component
const textInput = {
  inject: ['emitter'],
  props: {
    field: { required: true },
    data: { required: true }
  },
  
  template: `
  <div class="form-control">
    <label>{{ field.label }}</label>
    <input class="input" type="text" @blur="fieldValueChanged" :placeholder="field.placeholder" v-model="data.value">
  </div>
  `,
  methods: {
    fieldValueChanged(){
      this.emitter.trigger('process');
    }
  },
};

// Textarea field component
const textareaInput = {
  inject: ['emitter'],
  props: {
    field: { required: true },
    data: { required: true }
  },
  
  template: `
  <div class="form-control">
    <label>{{ field.label }}</label>
    <textarea class="input" type="text" @blur="fieldValueChanged" :placeholder="field.placeholder" v-model="data.value"></textarea>
  </div>
  `,
  methods: {
    fieldValueChanged(){
      this.emitter.trigger('process');
    }
  },
};

// Checkbox input field component
const checkboxInput = {
  inject: ['emitter'],
  props: {
    field: { required: true },
    data: { required: true }
  },
  
  template: `
  <div class="form-control">
    <label>{{ field.label }}</label>
    <label class="input">
      <input type="checkbox" v-model="data.value">
      {{ field.placeholder }}
    </label>
  </div>
  `
};

// Checkbox input field component
const selectInput = {
  inject: ['emitter'],
  props: {
    field: { required: true },
    data: { required: true }
  },
  components: {
    Multiselect: window.VueMultiselect.default
  },
  
  template: `
  <div class="form-control">
    <label>{{ field.label }}</label>
    <multiselect v-model="data.default_value" @input="fieldValueChanged" :options="data.value" label="name" track-by="key" 
      :searchable="false" :close-on-select="true" :show-labels="false" placeholder="Select a field">
    </multiselect>
  </div>
  `,
  methods: {
    fieldValueChanged(value){
      this.emitter.trigger('process');
    }
  }

};

// Dynamic form component
const vForm = {
  template: `
  <div class="VueForm">
    <template v-for="field in schema.fields">
      <component :is="field.type" :field="field" :data.sync="data[field.name]"></component>
    </template>
  </div>
  `,
  
  components: {
    textInput: textInput,
    checkboxInput: checkboxInput,
    selectInput:selectInput,
    textareaInput: textareaInput,
  },
  
  props: {
    schema: { required: true },
    data: { required: true }
  }
};

Vue.component('v-form', vForm);

// New add action for setting default values of fields
var VueFieldDefaultValueControl = {
  components: {
    // Component included for Multiselect.
    Multiselect: window.VueMultiselect.default
  },
  props: ['getData', 'putData', 'emitter'],
  provide() {
    return {
      emitter: this.emitter
    }
  },
  data(){
    return{
      type: drupalSettings.ifthenelserule.nodes.setFieldValueAction.type,
      class: drupalSettings.ifthenelserule.nodes.setFieldValueAction.class,
      options: [],
      form_fields: [],
      value: [],
      formSchema: {
        fields: []      
      },
      formData: {
      }
    }
  },
  template: `<div class="fields-container">
    <div class="entity-select">
      <label class="typo__label">Select a Field</label>
      <multiselect v-model="value" :options="options" @input="fieldValueChanged" label="name" track-by="code" 
      :searchable="false" :close-on-select="true" :show-labels="false" placeholder="Select a field">
      </multiselect>
      <v-form :schema="formSchema" :data="formData"></v-form>
    </div>
  </div>`,

  methods: {
    fieldValueChanged(value){
      //Triggered when selecting an field.
      // Setting dynamic field values for the form.
      var selected_entity = '';
      var selected_bundle = '';
      jQuery.each(editor.nodes,function(key,condition_node){
        if(condition_node.data.type == 'condition' && typeof condition_node.data.selected_entity != 'undefined' && typeof condition_node.data.selected_bundle != 'undefined'){
          selected_entity = condition_node.data.selected_entity.value;
          selected_bundle = condition_node.data.selected_bundle.value;      
        }
      });
      
      this.formSchema.fields = [];
      this.formData = {};
      if(value != null){
        var postData = [];
        postData = {entity: selected_entity, bundle: selected_bundle, field_name: value.code};
        Vue.http.post(drupalSettings.path.baseUrl+'ifthenelse/fetch-field-info',postData).then((response) => {
          // get body data
          var result = JSON.parse(response.bodyText);
          
  
          if(typeof result != 'undefined'){
            this.formSchema.fields.push({
              type: result.type,
              name: result.field_name,
              label: result.field_label,
            });
  
            this.formData = {
              [result.field_name]: {
                value: result.value,
              }
            }
          }
  
          this.putData('formSchema',this.formSchema);
          this.putData('formData',this.formData);
  
        }, (response) => {
          alert('There was an error processing your request.');
        });        
      }

      var selectedOptions = [];
      selectedOptions.push({name: value.name, code: value.code});
      
      this.putData('form_fields',selectedOptions);
      this.emitter.trigger('process');
    },    
  },

  mounted(){
    // initialize variable for data
    this.putData('type',drupalSettings.ifthenelserule.nodes.setFieldValueAction.type);
    this.putData('class',drupalSettings.ifthenelserule.nodes.setFieldValueAction.class);

    //setting values of selected fields when rule edit page loads.
    var get_form_fields = this.getData('form_fields');
    if(typeof get_form_fields != 'undefined'){
      this.value = this.getData('form_fields');
    }else{
      this.putData('form_fields',[]);
    }

    var get_formSchema = this.getData('formSchema');
    if(typeof get_formSchema != 'undefined'){
      this.formSchema = this.getData('formSchema');
    }else{
      this.putData('formSchema',[]);
    }

    var get_formData = this.getData('formData');
    if(typeof get_formData != 'undefined'){
      this.formData = this.getData('formData');
    }else{
      this.putData('formData',{});
    }
  },
  created() {
    if(drupalSettings.ifthenelserule.form_fields){
      //setting list of all fields for a form when rule edit page loads.
      this.options = JSON.parse(drupalSettings.ifthenelserule.form_fields);
    }
  }
}

class FieldDefaultValueControl extends Rete.Control {
  constructor(emitter, key, readonly) {
    super(key);
    this.component = VueFieldDefaultValueControl;
    this.props = { emitter, ikey: key, readonly };
  }

  //setting list value of fields. Used when changing entity or bundle value in condition node.
  setOptions(options) {
    this.vueContext.options = options;
  }

  //resetting selected fields value when changing the entity or bundle value in codition node.
  setValue(value){
    this.vueContext.value = value;
  }
}

class FieldDefaultValueComponent extends Rete.Component {

  constructor(){
      super("Action: Set Value of field");
  }

  builder(node) {
      var conditionInput = new Rete.Input('execute', "", conditionSocket);
      var formInput = new Rete.Input('form', "", formObjectSocket);

      node.addControl(new FieldDefaultValueControl(this.editor, 'setDefaultFieldValue'));

      return node.addInput(conditionInput).addInput(formInput);
  }

  worker(node, inputs, outputs) {
    outputs['form_fields'] = node.data.value;
  }
}

var components = [new FieldDefaultValueComponent()];	

components.map(c => {
  editor.register(c);
  engine.register(c);
});