<?php

namespace Drupal\if_then_else\core\Actions\SetFieldDefaultValue;

use Drupal\if_then_else\core\IfthenelseActionInterface;

/**
 * Class defined to execute make fields required action node.
 */
class SetFieldDefaultValue implements IfthenelseActionInterface {

  /**
   * All node Inputs.
   *
   * @var array
   */
  public $inputs;

  /**
   * All node Outputs.
   *
   * @var array
   */
  private $output;

  /**
   * Current node object data.
   *
   * @var array
   */
  private $data;

  /**
   * Ifthenelserule constructor.
   *
   * @param object $data
   *   Node data object.
   */
  public function __construct($data) {
    $this->data = $data;
  }

  /**
   * Function to set inputs.
   *
   * @param array $inputs
   *   Array of all inputs to node.
   */
  public function setInputs(array $inputs) {
    $this->inputs = $inputs;
  }

  /**
   * Get node output.
   *
   * @return output
   *   Returns output of current node set by process function.
   */
  public function getOutput() {
    return $this->output;
  }

  /**
   * Process inputs and set output.
   */
  public function process() {
    $data = $this->data;
    $this->setValueOfField($data->form_fields, $data->formData, $this->inputs['form']);
  }

  /**
   * Make selected fields required.
   *
   * @param array $form_fields
   *   List of selected fields to make required.
   * @param object $formData
   *   Action node field value.
   * @param object $form
   *   Form object from hook form alter.
   */
  public function setValueOfField(array $form_fields, $formData, &$form) {
    foreach ($form_fields as $field) {
      $field_code = $field->code;
      if (isset($form[$field_code])) {
        if ($form[$field_code]['#type'] == 'container') {
          if (isset($form[$field_code]['widget'])) {
            if (isset($form[$field_code]['widget']['#type'])) {
              if (isset($form[$field_code]['widget']['#type']) == 'select') {
                $form[$field_code]['widget']['#default_value'] = [$formData->{$field_code}->default_value->key];
              }
            }
            else {
              foreach ($form[$field_code]['widget'] as $k => $value) {
                if (strpos($k, '#') !== FALSE) {
                  // Skip all keys which have #.
                  continue;
                }

                if (isset($value['#type']) && $value['#type'] == 'checkbox') {
                  // Skip all fields which are single checkboxes.
                  continue;
                }

                if ($value['#type'] == 'text_format') {
                  $form[$field_code]['widget'][$k]['#default_value'] = $formData->{$field_code}->value;
                }
                else {
                  $form[$field_code]['widget'][$k]['value']['#default_value'] = $formData->{$field_code}->value;
                }
              }
            }
          }
        }
        else {
          $form[$field_code]['#default_value'] = $formData->{$field_code}->value;
        }
      }
    }
  }

}
