<?php

namespace Drupal\if_then_else\core\Conditions\IfthenelseFormAlterCondition;

use Drupal\if_then_else\core\IfthenelseConditionInterface;
use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormInterface;

/**
 * Class defined to process make all fields required action.
 */
class IfthenelseFormAlterCondition implements IfthenelseConditionInterface {

  /**
   * All node Inputs.
   *
   * @var array
   */
  private $inputs;

  /**
   * All node Outputs.
   *
   * @var array
   */
  private $output;

  /**
   * Current node object data.
   *
   * @var array
   */
  private $data;

  /**
   * Ifthenelserule constructor.
   *
   * @param object $data
   *   Node data object.
   */
  public function __construct($data) {
    $this->data = $data;
  }

  /**
   * Function to set inputs.
   *
   * @param array $inputs
   *   Array of all inputs to node.
   */
  public function setInputs(array $inputs) {
    $this->inputs = $inputs;
  }

  /**
   * Get node output.
   *
   * @return output
   *   Returns output of current node set by process function.
   */
  public function getOutput() {
    return $this->output;
  }

  /**
   * Process inputs and set output.
   */
  public function process() {
    $form_id_matched = FALSE;
    $data = $this->data;
    $form_state = $this->inputs['formState'];
    $form_id = $form_state->getBuildInfo()['form_id'];

    if (isset($data->selected_entity->value) && $data->selected_entity->value == 'other_form') {
      // It is a form class so get the formId using formclass.
      $form_state = new FormState();
      $formId_from_class = \Drupal::formBuilder()->getFormId($data->otherFormClass, $form_state);
      if ($formId_from_class == $form_id) {
        $form_id_matched = TRUE;
      }
    }
    elseif (isset($data->selected_entity->value) && $data->selected_entity->value != 'other_form') {
      // It is content entity so get the formid using it.
      $form_object = $form_state->getFormObject();

      // Checking if form is a entity form.
      if (method_exists($form_object, 'getEntity')) {
        $form_entity_id = $form_state->getFormObject()->getEntity()->getEntityTypeId();
        $form_bundle_id = $form_state->getFormObject()->getEntity()->bundle();

        if ($form_entity_id == $data->selected_entity->value && $form_bundle_id == $data->selected_bundle->value) {
          $form_id_matched = TRUE;
        }
      }
    }
    $this->output['execute'] = $form_id_matched;
  }

  /**
   * Check if form class is valid.
   *
   * @param string $form_class
   *   Form Class name to be validated.
   *
   * @return bool
   *   Return if form class is valid or not.
   */
  public function validateFormClass($form_class) {
    if (is_string($form_class) && class_exists($form_class)) {
      // Generating class object from class string name to comapre
      // if it is instance of FormInterface.
      $other_form_class = \Drupal::classResolver($form_class);
      if (!is_object($other_form_class) || !($other_form_class instanceof FormInterface)) {
        return FALSE;
      }
      else {
        return TRUE;
      }
    }
    else {
      return FALSE;
    }
  }

}
